"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BunPackageManager = void 0;
const path = require("path");
const base_package_manager_1 = require("./base-package-manager");
const decorators_1 = require("./common/decorators");
const constants_1 = require("./constants");
const _ = require("lodash");
const yok_1 = require("./common/yok");
class BunPackageManager extends base_package_manager_1.BasePackageManager {
    constructor($childProcess, $errors, $fs, $hostInfo, $logger, $httpClient, $pacoteService) {
        super($childProcess, $fs, $hostInfo, $pacoteService, "bun");
        this.$errors = $errors;
        this.$logger = $logger;
        this.$httpClient = $httpClient;
    }
    async install(packageName, pathToSave, config) {
        if (config.disableNpmInstall) {
            return;
        }
        if (config.ignoreScripts) {
            config["ignore-scripts"] = true;
        }
        const packageJsonPath = path.join(pathToSave, "package.json");
        const jsonContentBefore = this.$fs.readJson(packageJsonPath);
        const flags = this.getFlagsString(config, true);
        let params = ["install"];
        const isInstallingAllDependencies = packageName === pathToSave;
        if (!isInstallingAllDependencies) {
            params.push(packageName);
        }
        params = params.concat(flags);
        const cwd = pathToSave;
        try {
            const result = await this.processPackageManagerInstall(packageName, params, { cwd, isInstallingAllDependencies });
            return result;
        }
        catch (err) {
            // Revert package.json contents to preserve valid state
            this.$fs.writeJson(packageJsonPath, jsonContentBefore);
            throw err;
        }
    }
    async uninstall(packageName, config, cwd) {
        const flags = this.getFlagsString(config, false);
        return this.$childProcess.exec(`bun remove ${packageName} ${flags}`, {
            cwd,
        });
    }
    // Bun does not have a `view` command; use npm.
    async view(packageName, config) {
        const wrappedConfig = _.extend({}, config, { json: true }); // always require view response as JSON
        const flags = this.getFlagsString(wrappedConfig, false);
        let viewResult;
        try {
            viewResult = await this.$childProcess.exec(`npm view ${packageName} ${flags}`);
        }
        catch (e) {
            this.$errors.fail(e.message);
        }
        try {
            return JSON.parse(viewResult);
        }
        catch (err) {
            return null;
        }
    }
    // Bun does not have a `search` command; use npm.
    async search(filter, config) {
        const flags = this.getFlagsString(config, false);
        return this.$childProcess.exec(`npm search ${filter.join(" ")} ${flags}`);
    }
    async searchNpms(keyword) {
        // Bugs with npms.io:
        // 1. API returns no results when a valid package name contains @ or /
        //    even if using encodeURIComponent().
        // 2. npms.io's API no longer returns updated results; see
        //    https://github.com/npms-io/npms-api/issues/112. Better to switch to
        //    https://registry.npmjs.org/<query>
        const httpRequestResult = await this.$httpClient.httpRequest(`https://api.npms.io/v2/search?q=keywords:${keyword}`);
        const result = JSON.parse(httpRequestResult.body);
        return result;
    }
    // Bun does not have a command analogous to `npm config get registry`; Bun
    // uses `bunfig.toml` to define custom registries.
    // - TODO: read `bunfig.toml`, if it exists, and return the registry URL.
    async getRegistryPackageData(packageName) {
        const registry = await this.$childProcess.exec(`npm config get registry`);
        const url = registry.trim() + packageName;
        this.$logger.trace(`Trying to get data from npm registry for package ${packageName}, url is: ${url}`);
        const responseData = (await this.$httpClient.httpRequest(url)).body;
        this.$logger.trace(`Successfully received data from npm registry for package ${packageName}. Response data is: ${responseData}`);
        const jsonData = JSON.parse(responseData);
        this.$logger.trace(`Successfully parsed data from npm registry for package ${packageName}.`);
        return jsonData;
    }
    async getCachePath() {
        const cachePath = await this.$childProcess.exec(`bun pm cache`);
        return path.join(cachePath.trim(), constants_1.CACACHE_DIRECTORY_NAME);
    }
}
exports.BunPackageManager = BunPackageManager;
__decorate([
    (0, decorators_1.exported)("bun")
], BunPackageManager.prototype, "install", null);
__decorate([
    (0, decorators_1.exported)("bun")
], BunPackageManager.prototype, "uninstall", null);
__decorate([
    (0, decorators_1.exported)("bun")
], BunPackageManager.prototype, "view", null);
__decorate([
    (0, decorators_1.exported)("bun")
], BunPackageManager.prototype, "search", null);
__decorate([
    (0, decorators_1.cache)()
], BunPackageManager.prototype, "getCachePath", null);
yok_1.injector.register("bun", BunPackageManager);
//# sourceMappingURL=bun-package-manager.js.map