"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RemovePluginCommand = void 0;
const _ = require("lodash");
const yok_1 = require("../../common/yok");
class RemovePluginCommand {
    constructor($pluginsService, $errors, $logger, $projectData) {
        this.$pluginsService = $pluginsService;
        this.$errors = $errors;
        this.$logger = $logger;
        this.$projectData = $projectData;
        this.allowedParameters = [];
        this.$projectData.initializeProjectData();
    }
    async execute(args) {
        return this.$pluginsService.remove(args[0], this.$projectData);
    }
    async canExecute(args) {
        if (!args[0]) {
            this.$errors.failWithHelp("You must specify plugin name.");
        }
        let pluginNames = [];
        try {
            // try installing the plugins, so we can get information from node_modules about their native code, libs, etc.
            const installedPlugins = await this.$pluginsService.getAllInstalledPlugins(this.$projectData);
            pluginNames = installedPlugins.map((pl) => pl.name);
        }
        catch (err) {
            this.$logger.trace("Error while installing plugins. Error is:", err);
            pluginNames = _.keys(this.$projectData.dependencies);
        }
        const pluginName = args[0].toLowerCase();
        if (!_.some(pluginNames, (name) => name.toLowerCase() === pluginName)) {
            this.$errors.fail(`Plugin "${pluginName}" is not installed.`);
        }
        return true;
    }
}
exports.RemovePluginCommand = RemovePluginCommand;
yok_1.injector.registerCommand("plugin|remove", RemovePluginCommand);
//# sourceMappingURL=remove-plugin.js.map