"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UpdatePluginCommand = void 0;
const _ = require("lodash");
const yok_1 = require("../../common/yok");
class UpdatePluginCommand {
    constructor($pluginsService, $projectData, $errors) {
        this.$pluginsService = $pluginsService;
        this.$projectData = $projectData;
        this.$errors = $errors;
        this.allowedParameters = [];
        this.$projectData.initializeProjectData();
    }
    async execute(args) {
        let pluginNames = args;
        if (!pluginNames || args.length === 0) {
            const installedPlugins = await this.$pluginsService.getAllInstalledPlugins(this.$projectData);
            pluginNames = installedPlugins.map((p) => p.name);
        }
        for (const pluginName of pluginNames) {
            await this.$pluginsService.remove(pluginName, this.$projectData);
            await this.$pluginsService.add(pluginName, this.$projectData);
        }
    }
    async canExecute(args) {
        if (!args || args.length === 0) {
            return true;
        }
        const installedPlugins = await this.$pluginsService.getAllInstalledPlugins(this.$projectData);
        const installedPluginNames = installedPlugins.map((pl) => pl.name);
        const pluginName = args[0].toLowerCase();
        if (!_.some(installedPluginNames, (name) => name.toLowerCase() === pluginName)) {
            this.$errors.fail(`Plugin "${pluginName}" is not installed.`);
        }
        return true;
    }
}
exports.UpdatePluginCommand = UpdatePluginCommand;
yok_1.injector.registerCommand("plugin|update", UpdatePluginCommand);
//# sourceMappingURL=update-plugin.js.map