"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IOSDeviceDiscovery = void 0;
const device_discovery_1 = require("./device-discovery");
const ios_device_1 = require("../ios/device/ios-device");
const yok_1 = require("../../yok");
class IOSDeviceDiscovery extends device_discovery_1.DeviceDiscovery {
    constructor($injector, $logger, $mobileHelper, $iosDeviceOperations, $devicePlatformsConstants) {
        super();
        this.$injector = $injector;
        this.$logger = $logger;
        this.$mobileHelper = $mobileHelper;
        this.$iosDeviceOperations = $iosDeviceOperations;
        this.$devicePlatformsConstants = $devicePlatformsConstants;
    }
    async startLookingForDevices(options) {
        this.$logger.trace("Options for ios-device-discovery", options);
        if (options &&
            options.platform &&
            (!this.$mobileHelper.isApplePlatform(options.platform) ||
                options.emulator)) {
            return;
        }
        if (this.$mobileHelper.isvisionOSPlatform(options.platform)) {
            // look for ios devices - same logic.
            options.platform = this.$devicePlatformsConstants.iOS;
        }
        await this.$iosDeviceOperations.startLookingForDevices((deviceInfo) => {
            const device = this.createDevice(deviceInfo);
            this.addDevice(device);
        }, (deviceInfo) => {
            const currentDevice = this.getDevice(deviceInfo.deviceId);
            if (currentDevice) {
                const device = this.createDevice(deviceInfo);
                this.updateDeviceInfo(device);
            }
            else {
                const device = this.createDevice(deviceInfo);
                this.addDevice(device);
            }
        }, (deviceInfo) => {
            this.removeDevice(deviceInfo.deviceId);
        }, options);
    }
    createDevice(deviceActionInfo) {
        const device = this.$injector.resolve(ios_device_1.IOSDevice, {
            deviceActionInfo: deviceActionInfo,
        });
        return device;
    }
}
exports.IOSDeviceDiscovery = IOSDeviceDiscovery;
yok_1.injector.register("iOSDeviceDiscovery", IOSDeviceDiscovery);
//# sourceMappingURL=ios-device-discovery.js.map