"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PnpmPackageManager = void 0;
const path = require("path");
const _ = require("lodash");
const base_package_manager_1 = require("./base-package-manager");
const decorators_1 = require("./common/decorators");
const constants_1 = require("./constants");
const yok_1 = require("./common/yok");
class PnpmPackageManager extends base_package_manager_1.BasePackageManager {
    constructor($childProcess, $errors, $fs, $hostInfo, $httpClient, $logger, $pacoteService) {
        super($childProcess, $fs, $hostInfo, $pacoteService, "pnpm");
        this.$errors = $errors;
        this.$httpClient = $httpClient;
        this.$logger = $logger;
    }
    async install(packageName, pathToSave, config) {
        if (config.disableNpmInstall) {
            return;
        }
        delete config.dev; // temporary fix for unsupported yarn flag
        if (config.ignoreScripts) {
            config["ignore-scripts"] = true;
        }
        const packageJsonPath = path.join(pathToSave, "package.json");
        const jsonContentBefore = this.$fs.readJson(packageJsonPath);
        const flags = this.getFlagsString(config, true);
        // With pnpm we need to install as "flat" or some imports wont be found
        let params = ["i", "--shamefully-hoist"];
        const isInstallingAllDependencies = packageName === pathToSave;
        if (!isInstallingAllDependencies) {
            params.push(packageName);
        }
        params = params.concat(flags);
        const cwd = pathToSave;
        try {
            const result = await this.processPackageManagerInstall(packageName, params, { cwd, isInstallingAllDependencies });
            return result;
        }
        catch (e) {
            this.$fs.writeJson(packageJsonPath, jsonContentBefore);
            throw e;
        }
    }
    uninstall(packageName, config, cwd) {
        // pnpm does not want save option in remove. It saves it by default
        delete config["save"];
        const flags = this.getFlagsString(config, false);
        return this.$childProcess.exec(`pnpm remove ${packageName} ${flags}`, {
            cwd,
        });
    }
    async view(packageName, config) {
        const wrappedConfig = _.extend({}, config, { json: true });
        const flags = this.getFlagsString(wrappedConfig, false);
        let viewResult;
        try {
            viewResult = await this.$childProcess.exec(`pnpm info ${packageName} ${flags}`);
        }
        catch (e) {
            this.$errors.fail(e.message);
        }
        try {
            return JSON.parse(viewResult);
        }
        catch (err) {
            return null;
        }
    }
    search(filter, config) {
        const flags = this.getFlagsString(config, false);
        return this.$childProcess.exec(`pnpm search ${filter.join(" ")} ${flags}`);
    }
    async searchNpms(keyword) {
        const httpRequestResult = await this.$httpClient.httpRequest(`https://api.npms.io/v2/search?q=keywords:${keyword}`);
        const result = JSON.parse(httpRequestResult.body);
        return result;
    }
    async getRegistryPackageData(packageName) {
        const registry = await this.$childProcess.exec(`pnpm config get registry`);
        const url = `${registry.trim()}/${packageName}`;
        this.$logger.trace(`Trying to get data from pnpm registry for package ${packageName}, url is: ${url}`);
        const responseData = (await this.$httpClient.httpRequest(url)).body;
        this.$logger.trace(`Successfully received data from pnpm registry for package ${packageName}. Response data is: ${responseData}`);
        const jsonData = JSON.parse(responseData);
        this.$logger.trace(`Successfully parsed data from pnpm registry for package ${packageName}.`);
        return jsonData;
    }
    async getCachePath() {
        const cachePath = await this.$childProcess.exec(`pnpm config get cache`);
        return path.join(cachePath.trim(), constants_1.CACACHE_DIRECTORY_NAME);
    }
}
exports.PnpmPackageManager = PnpmPackageManager;
__decorate([
    (0, decorators_1.exported)("pnpm")
], PnpmPackageManager.prototype, "install", null);
__decorate([
    (0, decorators_1.exported)("pnpm")
], PnpmPackageManager.prototype, "uninstall", null);
__decorate([
    (0, decorators_1.exported)("pnpm")
], PnpmPackageManager.prototype, "view", null);
__decorate([
    (0, decorators_1.exported)("pnpm")
], PnpmPackageManager.prototype, "search", null);
__decorate([
    (0, decorators_1.exported)("pnpm")
], PnpmPackageManager.prototype, "getRegistryPackageData", null);
__decorate([
    (0, decorators_1.exported)("pnpm")
], PnpmPackageManager.prototype, "getCachePath", null);
yok_1.injector.register("pnpm", PnpmPackageManager);
//# sourceMappingURL=pnpm-package-manager.js.map