"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.IPService = void 0;
const decorators_1 = require("../common/decorators");
const yok_1 = require("../common/yok");
class IPService {
    constructor($httpClient, $logger) {
        this.$httpClient = $httpClient;
        this.$logger = $logger;
    }
    async getCurrentIPv4Address() {
        const ipAddress = (await this.getIPAddressFromServiceReturningJSONWithIPProperty("https://api.myip.com")) ||
            (await this.getIPAddressFromIpifyOrgAPI()) ||
            null;
        return ipAddress;
    }
    async getIPAddressFromServiceReturningJSONWithIPProperty(apiEndpoint) {
        let ipAddress = null;
        try {
            const response = await this.$httpClient.httpRequest({
                method: "GET",
                url: apiEndpoint,
                timeout: IPService.GET_IP_TIMEOUT,
            });
            this.$logger.trace(`${apiEndpoint} returns ${response.body}`);
            const jsonData = JSON.parse(response.body);
            ipAddress = jsonData.ip;
        }
        catch (err) {
            this.$logger.trace(`Unable to get information about current IP Address from ${apiEndpoint} Error is:`, err);
        }
        return ipAddress;
    }
    async getIPAddressFromIpifyOrgAPI() {
        // https://www.ipify.org/
        const ipifyOrgAPIEndpoint = "https://api.ipify.org";
        let ipAddress = null;
        try {
            const response = await this.$httpClient.httpRequest({
                method: "GET",
                url: ipifyOrgAPIEndpoint,
                timeout: IPService.GET_IP_TIMEOUT,
            });
            this.$logger.trace(`${ipifyOrgAPIEndpoint} returns ${response.body}`);
            ipAddress = (response.body || "").toString();
        }
        catch (err) {
            this.$logger.trace(`Unable to get information about current IP Address from ${ipifyOrgAPIEndpoint} Error is:`, err);
        }
        return ipAddress;
    }
}
exports.IPService = IPService;
IPService.GET_IP_TIMEOUT = 1000;
__decorate([
    (0, decorators_1.cache)()
], IPService.prototype, "getCurrentIPv4Address", null);
yok_1.injector.register("ipService", IPService);
//# sourceMappingURL=ip-service.js.map