"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeviceLiveSyncServiceBase = void 0;
const decorators_1 = require("../../common/decorators");
const path = require("path");
const _ = require("lodash");
const decorators_2 = require("../../common/decorators");
class DeviceLiveSyncServiceBase {
    constructor(platformsDataService, device) {
        this.platformsDataService = platformsDataService;
        this.device = device;
    }
    canExecuteFastSync(liveSyncResult, filePath, projectData, platform) {
        const fastSyncFileExtensions = this.getFastLiveSyncFileExtensions(platform, projectData);
        return (liveSyncResult.useHotModuleReload ||
            _.includes(fastSyncFileExtensions, path.extname(filePath)));
    }
    canExecuteFastSyncForPaths(liveSyncResult, localToDevicePaths, projectData, platform) {
        return !_.some(localToDevicePaths, (localToDevicePath) => !this.canExecuteFastSync(liveSyncResult, localToDevicePath.getLocalPath(), projectData, this.device.deviceInfo.platform));
    }
    getFastLiveSyncFileExtensions(platform, projectData) {
        const platformData = this.platformsDataService.getPlatformData(platform, projectData);
        const fastSyncFileExtensions = DeviceLiveSyncServiceBase.FAST_SYNC_FILE_EXTENSIONS.concat(platformData.fastLivesyncFileExtensions);
        return fastSyncFileExtensions;
    }
    async transferFiles(deviceAppData, localToDevicePaths, projectFilesPath, projectData, liveSyncDeviceDescriptor, options) {
        let transferredFiles = [];
        if (options.isFullSync) {
            transferredFiles = await this.device.fileSystem.transferDirectory(deviceAppData, localToDevicePaths, projectFilesPath);
        }
        else {
            transferredFiles = await this.device.fileSystem.transferFiles(deviceAppData, localToDevicePaths);
        }
        return transferredFiles;
    }
    async finalizeSync(liveSyncInfo, projectData) {
        //implement in case a sync point for all remove/create operation is needed
        return {
            didRefresh: true,
            operationId: "",
        };
    }
}
exports.DeviceLiveSyncServiceBase = DeviceLiveSyncServiceBase;
DeviceLiveSyncServiceBase.FAST_SYNC_FILE_EXTENSIONS = [".css", ".xml", ".html"];
__decorate([
    (0, decorators_1.cache)()
], DeviceLiveSyncServiceBase.prototype, "getFastLiveSyncFileExtensions", null);
__decorate([
    (0, decorators_2.performanceLog)()
], DeviceLiveSyncServiceBase.prototype, "transferFiles", null);
//# sourceMappingURL=device-livesync-service-base.js.map